#!/bin/ksh
# @version $Id: statusget.sh,v 2.4.1.1 2003/12/12 22:24:22 fredo Exp $
# @copyright 2003 
# call : statusget.sh
# return changed status back to the main BBDISPLAY server or hypervision
# since last time check for all hosts specified in the config file 
# for the specified services
#
# This is compatible with bbmap if used.
# One can use this for only making bbrelay in a more light way
# than standard one from BB4. Here are the main ideas :
#
# INSTALLATION :
# on sub BBDISPLAY servers (only BBDISPLAYs)
# 1) copy statusget.sh and statusloop.sh in /usr/local/bb4/ext/
# 2) modify statusget.sh between BEGIN CONFIGURATION and END CONFIGURATION
# 3) modify statusloop.sh if needed (for instance the sleep delay)
# 4) create or copy LASTCOUNT.txt in LASTCHKDIR (see below in CONFIGURATION)
#    with 0 inside (nothing more) and write access for bb4 user
# 5) create LCKCONFIG.txt (see CONFIGFILE in CONFIGURATION) with the following
#		server1:service1,service2,service3,...
#		server2:service1,service4,...
#		server3:...
#	where serverx = name of server as in bbhost and servicex as in bbweb page
#	(not as in bbhost) for instance service = conn or oracle or cpu...
# 5) add in bbrun.sh or bbdisplay start file the command to launch statusloop.sh
#     automaticaly when starting bbrun.sh
#    or start it manually as bb4 user
#
# on main BBDISPLAY server (BBRELAY one or hypervision)
# 1) check security in bb4/etc/security to allow sub-BBDISPLAYs to send status to this one
# 2) using bbgen, create your own web page by creating a particular bb-host file
# and running such a command
#		BBHOSTS=yourfile
#		export BBHOSTS
#		TEMPLATE="your_template_if_different_than_standard"
#		${BBHOME}/bin/bbgen --template=${TEMPLATE} --embedded > your_page.html
# this will produce web page as needed


ulimit -t 5

#BBHOME=/usr/local/bb4; export BBHOME   # FOR TESTING
if test "$BBHOME" = ""
then
    echo "BBHOME is not set... exiting"
    exit 1
fi

if test ! "$BBTMP"                      # GET DEFINITIONS IF NEEDED
then
    # echo "*** LOADING BBDEF ***"
    . $BBHOME/etc/bbdef.sh          # INCLUDE STANDARD DEFINITIONS
fi

# BBVAR="/usr/local/bbvar"

######## BEGIN CONFIGURATION

# PUT the IP Address of the final BBDISPLAY (hypervision)
FINALHOST="x.x.x.x"
# DIRECTORY for writing files of temporary status and counter
LASTCHKDIR="/usr/local/bb4/tmp/"
# Place where is configuration file
CONFIGFILE="/usr/local/bb4/etc/LCKCONFIG.txt"

######### END CONFIGURATION

# almost infinite validity of this test when sended
VALIDFOR=9999
# how many times before forcing status to be sent
RESUME=2000

MYDATE=`$DATE`
TMSTAMP=`date +%d%m%Y%H%M%S`
COUNT=`cat ${LASTCHKDIR}/LASTCOUNT.txt`
COUNT2=`$EXPR $COUNT + 1`
if [ $COUNT2 -gt $RESUME ]
then
    COUNT2=1
fi
echo $COUNT2 > ${LASTCHKDIR}/LASTCOUNT.txt

statuschange()
{
    cd ${BBVAR}/logs/
    SRCHOST=$1
    HOST2=`echo $1 | sed 's/\\./,/g'`
    SERVICES2=`echo $2 | sed 's/,/|/g'`
    lckred=""
    lckyellow=""
    lckpurple=""
    lckblue=""
    lckclear=""
    lckgreen=""
    redservice=`grep -l "^red" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`
    yellowservice=`grep -l "^yellow" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`
    purpleservice=`grep -l "^purple" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`
    blueservice=`grep -l "^blue" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`
    clearservice=`grep -l "^clear" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`
    greenservice=`grep -l "^green" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`

    for I in ${redservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  grep $I ${LASTCHKDIR}/${HOST2}.lck.red > /dev/null 2> /dev/null
          if [ $? -eq 1 ]
	      then
	      if [ "$lckred" = "" ]
		  then
		  lckred="$I"
	      else
		  lckred="$lckred%$I"
	      fi
	  fi
      fi
    done
    for I in ${yellowservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  grep $I ${LASTCHKDIR}/${HOST2}.lck.yellow > /dev/null 2> /dev/null
          if [ $? -eq 1 ]
	      then
	      if [ "$lckyellow" = "" ]
		  then
		  lckyellow="$I"
	      else
		  lckyellow="$lckyellow%$I"
	      fi
	  fi
      fi
    done
    for I in ${purpleservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  grep $I ${LASTCHKDIR}/${HOST2}.lck.purple > /dev/null 2> /dev/null
          if [ $? -eq 1 ]
	      then
	      if [ "$lckpurple" = "" ]
		  then
		  lckpurple="$I"
	      else
		  lckpurple="$lckpurple%$I"
	      fi
	  fi
      fi
    done
    for I in ${blueservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  grep $I ${LASTCHKDIR}/${HOST2}.lck.blue > /dev/null 2> /dev/null
          if [ $? -eq 1 ]
	      then
	      if [ "$lckblue" = "" ]
		  then
		  lckblue="$I"
	      else
		  lckblue="$lckblue%$I"
	      fi
	  fi
      fi
    done
    for I in ${clearservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  grep $I ${LASTCHKDIR}/${HOST2}.lck.clear > /dev/null 2> /dev/null
          if [ $? -eq 1 ]
	      then
	      if [ "$lckclear" = "" ]
		  then
		  lckclear="$I"
	      else
		  lckclear="$lckclear%$I"
	      fi
	  fi
      fi
    done
    for I in ${greenservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  grep $I ${LASTCHKDIR}/${HOST2}.lck.green > /dev/null 2> /dev/null
          if [ $? -eq 1 ]
	      then
	      if [ "$lckgreen" = "" ]
		  then
		  lckgreen="$I"
	      else
		  lckgreen="$lckgreen%$I"
	      fi
	  fi
      fi
    done

    echo ${redservice} > ${LASTCHKDIR}/${HOST2}.lck.red
    echo ${yellowservice} > ${LASTCHKDIR}/${HOST2}.lck.yellow
    echo ${purpleservice} > ${LASTCHKDIR}/${HOST2}.lck.purple
    echo ${blueservice} > ${LASTCHKDIR}/${HOST2}.lck.blue
    echo ${clearservice} > ${LASTCHKDIR}/${HOST2}.lck.clear
    echo ${greenservice} > ${LASTCHKDIR}/${HOST2}.lck.green
    if [ "${lckred}${lckyellow}${lckpurple}${lckblue}${lckclear}${lckgreen}" != "" ]
    then
# if debug wanted
#	echo ${TMSTAMP}:${SRCHOST}:${lckred}:${lckyellow}:${lckpurple}:${lckblue}:${lckclear}:${lckgreen} >> ${LASTCHKDIR}/LASTCHECK.lck
	CHANGE="${CHANGE} ${TMSTAMP}:${SRCHOST}:${lckred}:${lckyellow}:${lckpurple}:${lckblue}:${lckclear}:${lckgreen}"
    fi
}

statusnew()
{
    cd ${BBVAR}/logs/
    SRCHOST=$1
    HOST2=`echo $1 | sed 's/\\./,/g'`
    SERVICES2=`echo $2 | sed 's/,/|/g'`
    lckred=""
    lckyellow=""
    lckpurple=""
    lckblue=""
    lckclear=""
    lckgreen=""
    redservice=`grep -l "^red" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`
    yellowservice=`grep -l "^yellow" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`
    purpleservice=`grep -l "^purple" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`
    blueservice=`grep -l "^blue" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`
    clearservice=`grep -l "^clear" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`
    greenservice=`grep -l "^green" ${HOST2}.* | sed "s/${HOST2}\\.//g" 2> /dev/null`

    for I in ${redservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  if [ "$lckred" = "" ]
	      then
	      lckred="$I"
	  else
	      lckred="$lckred%$I"
	  fi
      fi
    done
    for I in ${yellowservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  if [ "$lckyellow" = "" ]
	      then
	      lckyellow="$I"
	  else
	      lckyellow="$lckyellow%$I"
	  fi
      fi
    done
    for I in ${purpleservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  if [ "$lckpurple" = "" ]
	      then
	      lckpurple="$I"
	  else
	      lckpurple="$lckpurple%$I"
	  fi
      fi
    done
    for I in ${blueservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  if [ "$lckblue" = "" ]
	      then
	      lckblue="$I"
	  else
	      lckblue="$lckblue%$I"
	  fi
      fi
    done
    for I in ${clearservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	then
	  if [ "$lckclear" = "" ]
	      then
	      lckclear="$I"
	  else
	      lckclear="$lckclear%$I"
	  fi
      fi
    done
    for I in ${greenservice}
    do
      echo $I | grep -E "${SERVICES2}" > /dev/null
      if [ $? -eq 0 ]
	  then
	  if [ "$lckgreen" = "" ]
	      then
	      lckgreen="$I"
	  else
	      lckgreen="$lckgreen%$I"
	  fi
      fi
    done

    echo ${redservice} > ${LASTCHKDIR}/${HOST2}.lck.red
    echo ${yellowservice} > ${LASTCHKDIR}/${HOST2}.lck.yellow
    echo ${purpleservice} > ${LASTCHKDIR}/${HOST2}.lck.purple
    echo ${blueservice} > ${LASTCHKDIR}/${HOST2}.lck.blue
    echo ${clearservice} > ${LASTCHKDIR}/${HOST2}.lck.clear
    echo ${greenservice} > ${LASTCHKDIR}/${HOST2}.lck.green
    if [ "${lckred}${lckyellow}${lckpurple}${lckblue}${lckclear}${lckgreen}" != "" ]
    then
# if debug wanted
#	echo ${TMSTAMP}:${SRCHOST}:${lckred}:${lckyellow}:${lckpurple}:${lckblue}:${lckclear}:${lckgreen} >> ${LASTCHKDIR}/LASTCHECK.lck
	CHANGE="${CHANGE} ${TMSTAMP}:${SRCHOST}:${lckred}:${lckyellow}:${lckpurple}:${lckblue}:${lckclear}:${lckgreen}"
    fi
}


# get which hosts and services
LISTE=`cat ${CONFIGFILE}`
CHANGE=""
export CHANGE
# generate HOST CHECK
if [ $COUNT2 -eq 1 ]
then
    for L in $LISTE
      do
      HOST=`echo $L | cut -f1 -d: | sed 's/\\./,/g'`
      SERVICES=`echo $L | cut -f2 -d:`
      statusnew ${HOST} ${SERVICES}
    done
else
    for L in $LISTE
      do
      HOST=`echo $L | cut -f1 -d: | sed 's/\\./,/g'`
      SERVICES=`echo $L | cut -f2 -d:`
      statuschange ${HOST} ${SERVICES}
    done
fi

if [ "$CHANGE" != "" ]
then
# separate hosts, then red, yellow, ..., then services
    for I in $CHANGE
    do
	HOST=`echo $I | cut -f2 -d: `
	for J in 3 4 5 6 7 8
	do
	    case $J in
		3)  COLOR="red" ;;
		4)  COLOR="yellow" ;;
		5)  COLOR="purple" ;;
		6)  COLOR="blue" ;;
		7)  COLOR="clear" ;;
		8)  COLOR="green" ;;
	    esac
	    EXPLOSE=`echo $I | cut -f${J} -d: | sed 's/%/ /g' `
	    for TEST in $EXPLOSE
	    do
# if debug wanted
#		echo $BB $FINALHOST "status+$VALIDFOR $HOST.$TEST $COLOR $MYDATE External Test"  >> ${LASTCHKDIR}/LASTCHECK.lck
		$BB $FINALHOST "status+$VALIDFOR $HOST.$TEST $COLOR $MYDATE #$BBWEBHOST$BBWEB"
	    done
	done
    done
fi
